Package: ZygosityPredictor
Type: Package
Title: Package for prediction of zygosity for variants/genes in NGS
        data
Version: 1.13.0
Date: 2025-07-28
Authors@R: c(
  person("Marco", "Rheinnecker", 
    email="marco.rheinnecker@dkfz-heidelberg.de", 
    role = c("aut", "cre"),
    comment=c(ORCID="0009-0009-7181-3977")), 
  person("Marc", "Ruebsam", 
    role = c("aut")),
  person("Daniel", "Huebschmann", 
    role = c("aut")),  
  person("Martina", "Froehlich", 
    role = c("aut")),
  person("Barbara", "Hutter", 
    role = c("aut"))
    )
Imports: GenomicAlignments, GenomicRanges, Rsamtools, IRanges,
        VariantAnnotation, DelayedArray, dplyr, stringr, purrr, tibble,
        methods, knitr, igraph, readr, stats, magrittr, rlang
License: GPL-2
Description: The ZygosityPredictor allows to predict how many copies of a gene are affected by small variants. In addition to the basic calculations of the affected copy number of a variant, the Zygosity-Predictor can integrate the influence of several variants on a gene and ultimately make a statement if and how many wild-type copies of the gene are left. This information proves to be of particular use in the context of translational medicine. For example, in cancer genomes, the Zygosity-Predictor can address whether unmutated copies of tumor-suppressor genes are present. Beyond this, it is possible to make this statement for all genes of an organism. The Zygosity-Predictor was primarily developed to handle SNVs and INDELs (later addressed as small-variants) of somatic and germline origin. In order not to overlook severe effects outside of the small-variant context, it has been extended with the assessment of large scale deletions, which cause losses of whole genes or parts of them.
RoxygenNote: 7.2.3
Encoding: UTF-8
biocViews: BiomedicalInformatics, FunctionalPrediction,
        SomaticMutation, GenePrediction
Depends: R (>= 4.3.0)
LazyData: false
Suggests: rmarkdown, testthat, BiocStyle
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/ZygosityPredictor
git_branch: devel
git_last_commit: 207b374
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:53:23 UTC; biocbuild
Author: Marco Rheinnecker [aut, cre] (ORCID:
    <https://orcid.org/0009-0009-7181-3977>),
  Marc Ruebsam [aut],
  Daniel Huebschmann [aut],
  Martina Froehlich [aut],
  Barbara Hutter [aut]
Maintainer: Marco Rheinnecker <marco.rheinnecker@dkfz-heidelberg.de>
