# Generated by roxygen2: do not edit by hand

export(ZP_ov)
export(aff_germ_copies)
export(aff_som_copies)
export(gene_ov)
export(predict_per_variant)
export(predict_zygosity)
importFrom(DelayedArray,rowRanges)
importFrom(GenomicAlignments,cigarRangesAlongQuerySpace)
importFrom(GenomicAlignments,cigarRangesAlongReferenceSpace)
importFrom(GenomicAlignments,end)
importFrom(GenomicAlignments,first)
importFrom(GenomicAlignments,granges)
importFrom(GenomicAlignments,last)
importFrom(GenomicAlignments,readGAlignmentPairs)
importFrom(GenomicAlignments,readGAlignments)
importFrom(GenomicAlignments,start)
importFrom(GenomicAlignments,width)
importFrom(GenomicRanges,"elementMetadata<-")
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,elementMetadata)
importFrom(GenomicRanges,end)
importFrom(GenomicRanges,seqnames)
importFrom(GenomicRanges,start)
importFrom(IRanges,IRanges)
importFrom(IRanges,mergeByOverlaps)
importFrom(IRanges,subsetByOverlaps)
importFrom(Rsamtools,ScanBamParam)
importFrom(Rsamtools,TabixFile)
importFrom(Rsamtools,scanBamFlag)
importFrom(Rsamtools,seqnamesTabix)
importFrom(VariantAnnotation,geno)
importFrom(VariantAnnotation,info)
importFrom(VariantAnnotation,readVcf)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,as_tibble)
importFrom(dplyr,between)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,desc)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_all)
importFrom(dplyr,mutate_at)
importFrom(dplyr,na_if)
importFrom(dplyr,nth)
importFrom(dplyr,pull)
importFrom(dplyr,relocate)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,summarize)
importFrom(dplyr,tally)
importFrom(dplyr,tibble)
importFrom(dplyr,ungroup)
importFrom(igraph,all_shortest_paths)
importFrom(igraph,graph_from_data_frame)
importFrom(knitr,kable)
importFrom(magrittr,"%>%")
importFrom(methods,is)
importFrom(purrr,compact)
importFrom(purrr,set_names)
importFrom(readr,write_tsv)
importFrom(rlang,ensym)
importFrom(stats,chisq.test)
importFrom(stats,na.omit)
importFrom(stringr,"%>%")
importFrom(stringr,str_detect)
importFrom(stringr,str_match)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_sub)
importFrom(tibble,as_tibble)
importFrom(tibble,column_to_rownames)
importFrom(tibble,tibble)
