#' copynumber object
#' @return Object containing somatic copy number abberations (sCNAs) of SeqC2 example case
#' @usage data(GR_SCNA)
#' @format ## `GR_SCNA`
#' GRanges object
"GR_SCNA"
#' somatic small variant object
#' @return Object containing somatic Indels and SNVs of SeqC2 example case
#' @usage data(GR_GERM_SMALL_VARS)
#' @format ## `GR_GERM_SMALL_VARS`
#' GRanges object
"GR_SOM_SMALL_VARS"
#' germline small variant object
#' @return Object containing germline Indels and SNVs of SeqC2 example case
#' @usage data(GR_SOM_SMALL_VARS)
#' @format ## `GR_SOM_SMALL_VARS`
#' GRanges object
"GR_GERM_SMALL_VARS"
#' germline small variant object
#' @return Object containing gene model of hg38
#' @usage data(GR_GENE_MODEL)
#' @format ## `GR_GENE_MODEL`
#' GRanges object
"GR_GENE_MODEL"
#' haploblocks 
#' @return Object containing haploblock annotations
#' @usage data(GR_HAPLOBLOCKS)
#' @format ## `GR_HAPLOBLOCKS`
#' GRanges object
"GR_HAPLOBLOCKS"