% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{GR_HAPLOBLOCKS}
\alias{GR_HAPLOBLOCKS}
\title{haploblocks}
\format{
## `GR_HAPLOBLOCKS`
GRanges object
}
\usage{
data(GR_HAPLOBLOCKS)
}
\value{
Object containing haploblock annotations
}
\description{
haploblocks
}
\keyword{datasets}
